import {register} from "@antv/x6-vue-shape";
import pointSetting from "@/util/graphSetting/pointSetting";

export const nodeLib = [
  {
    group: '数据输入',
    nodes: [
      {name: 'nodeInputTable', title: '输入DB表',},
      {name: 'sourceKafka', title: '输入kafka',},
      {name: 'SOURCELOCALFILE', title: '文件输入',},
      {name: 'SOURCE_FTP_SFTP', title: 'FTP/SFTPf文件输入',},
      {name: 'sourceHttp', title: '网络请求',},
      {name: 'sourceMysqlCdc', title: 'MysqlCdc',}
    ],
  },
  {
    group: '数据转换',
    nodes: [
      {name: 'TRANSFORMREPLACE', title: '文本替换'},
      {name: 'TRANSFORM_NULLREPLACE', title: 'NULL值替换'},
      {name: 'TRANSFORM_DISTINCT', title: '字段去重'},
      {name: 'TRANSFORM_COLUMNSELECT', title: '列选择'},
      {name: 'TRANSFORMCOLUMMERGE', title: '列合并'},
      {name: 'TRANSFORMCONDITION', title: '条件筛选'},
      {name: 'TRANSFORM_COLUMADD', title: '添加计算字段'},
      {name: 'TRANSFORM_TYPECONVERSION', title: '格式转换'},
      {name: 'TRANSFORM_PRIMAYKEY_ADD', title: '添加主键'},
      {name: 'TRANSFORM_ROWSTACK', title: '行堆叠'},
    ],
  },
  {
    group: '数据输出',
    nodes: [
      {name: 'nodeOutputTable', title: '输出DB表'},
      // {name: 'SINK_FTPFILE_SINK_SFTPFILE', title: '输出EXCEl'},
      // {name: 'SINKCLICKHOUSE', title: '输出表CLICKHOUSE'}
    ],
  }
]

const verifyList = []
const defaultVerify = () => Promise.resolve()

export default {
  install(app) {
    nodeLib.forEach(group => {
      let nodes = []
      group.nodes.forEach(node => {
        // 引入节点组件配置
        nodes.push({
          ...node,
          ...require(`./${node.name}/setting.js`).default,
        })
        let component = require(`./${node.name}/index.vue`).default
        let option = require(`./${node.name}/option.vue`).default
        // 注册节点组件
        register({
          shape: node.name,
          ...pointSetting,
          component,
        });
        app.component(node.name + 'Option', option)
        verifyList.push({
          name: node.name,
          func: option.methods ? (option.methods.verify || defaultVerify) : defaultVerify
        })
      })
      group.nodes = nodes
    })
  }
}

export const getVerify = name => verifyList.find(i => i.name === name).func
