import {Graph} from "@antv/x6";
import {ElMessage} from "element-plus";

// 生成并获取画布
export function getGraph(id, vm) {
  let graph = new Graph({
    container: document.getElementById(id),
    background: {color: "#f5f6fa",},
    panning: true,
    connecting: {
      allowBlank: false,
      allowLoop: false,
      allowNode: false,
      allowEdge: false,
      allowMulti: false,
      snap: true,
      validateConnection({sourcePort, targetPort}) {
        let targetCode = targetPort.split('-')[0]
        let targetNode = vm.nodeList.find(i => i.code === targetCode)
        let edges = vm.getEdges().filter(i => i.target)
        let targetNumber = edges.filter(i => i.target === targetCode).length
        let targetSetting = vm.allNodes.find(i => i.name === targetNode.name)
        if (targetSetting.inMax !== undefined) {
          if (targetNumber + 1 > targetSetting.inMax) {
            ElMessage({
              type: 'warning',
              message: `节点【${targetNode.title}】最大允许${targetSetting.inMax}条输入线`,
              grouping: true
            })
            return false
          }
        }
        return true
      },
      validateMagnet({cell, magnet}) {
        let type = magnet.getAttribute('port-group')
        if (type === 'in') {
          ElMessage({type: 'warning', message: '请从输出点开始连线'})
          return false
        }
        let sourceCode = cell.id
        let sourceNode = vm.nodeList.find(i => i.code === sourceCode)
        let edges = vm.getEdges().filter(i => i.target)
        let sourceNumber = edges.filter(i => i.source === sourceCode).length
        let sourceSetting = vm.allNodes.find(i => i.name === sourceNode.name)
        if (sourceSetting.outMax !== undefined) {
          if (sourceNumber + 1 > sourceSetting.outMax) {
            ElMessage({
              type: 'warning',
              message: `节点【${sourceNode.title}】最大允许${sourceSetting.outMax}条输出线`,
              grouping: true
            })
            return false
          }
        }
        return true
      },
      createEdge: () => graph.createEdge({shape: 'my-edge',}),
    },
  });
  graph.on('edge:mouseenter', ({edge}) => {
    edge.addTools(
      [{name: 'button-remove', args: {distance: '50%',},},],
    )
  })
  graph.on('edge:mouseleave', ({edge}) => {
    if (edge.hasTool('button-remove')) {
      edge.removeTool('button-remove')
    }
  })

  return graph
}
// 获取连接桩配置
export const getPortsByType = (type, nodeId) => {
  let ports = []
  switch (type) {
    case 'SOURCE':
      ports = [
        {id: `${nodeId}-out`, group: 'out',},
      ]
      break
    case 'SINK':
      ports = [
        {id: `${nodeId}-in`, group: 'in',},
      ]
      break
    default:
      ports = [
        {id: `${nodeId}-in`, group: 'in',},
        {id: `${nodeId}-out`, group: 'out',},
      ]
      break
  }
  return ports
}