import { nodeLib, getVerify } from "@/etlNodes";
import { ElMessage } from "element-plus";

/**
 *整理结构树 
 * @param {*}  
 * @param {*}  formList :[
  {
    "fromCode": "yiywtxiIQA",
    "toCode": "CjFvKaJwRi"
  },
  {
    "fromCode": "oMMIIWJEib",
    "toCode": "CjFvKaJwRi"
  },

]
 */
export const findSource = (edges) => {
  const nodeMap = {};
  edges.forEach((edge) => {
    const parentNode = nodeMap[edge.fromCode];
    const childNode = nodeMap[edge.toCode];

    if (parentNode && childNode) {
      if (!parentNode.children) {
        parentNode.children = [];
      }
      parentNode.children.push(childNode);
    } else if (parentNode) {
      const newChildNode = { id: edge.toCode, children: [] };
      parentNode.children = [newChildNode];
      nodeMap[edge.toCode] = newChildNode;
    } else if (childNode) {
      const newParentNode = { id: edge.fromCode, children: [childNode] };
      nodeMap[edge.fromCode] = newParentNode;
    } else {
      const newParentNode = { id: edge.fromCode, children: [] };
      const newChildNode = { id: edge.toCode, children: [] };
      newParentNode.children = [newChildNode];
      nodeMap[edge.fromCode] = newParentNode;
      nodeMap[edge.toCode] = newChildNode;
    }
  });
  // let allNode = []
  // Object.keys(nodeMap).map((key) => {
  //   if (nodeMap[key].children.length !==0) {
  //     allNode.push(nodeMap[key])
  //   }
  // });
  const rootNodes = Object.values(nodeMap).filter((node) => {
    return !edges.some((edge) => edge.toCode === node.id);
  });
  return rootNodes;
};
// 重置节点数据
/**
 *
 * @param {*} code
 * @param {*} nodeList
 * @param {*} tree :
 *
 */
export const resetNode = (code, nodeList, tree) => {
  if (tree.length === 0) return;
  function getAllNodesAfter(nodeId, rootNodes) {
    let specifiedNode = null;

    function findNode(node) {
      if (node.id === nodeId) {
        specifiedNode = node;
        return true;
      }

      if (node.children) {
        for (let i = 0; i < node.children.length; i++) {
          if (findNode(node.children[i])) {
            return true;
          }
        }
      }

      return false;
    }

    rootNodes.forEach((node) => {
      findNode(node);
    });

    function getAllChildren(node) {
      if (!node.children || node.children.length === 0) {
        return [];
      }

      let allChildren = [];

      node.children.forEach((child) => {
        allChildren.push(child);
        allChildren = allChildren.concat(getAllChildren(child));
      });

      return allChildren;
    }

    const allNodes = getAllChildren(specifiedNode); // 获取指定节点后的所有子节点

    return [specifiedNode, ...allNodes];
  }

  const nodesAfter = getAllNodesAfter(code, tree) || [];
  let allNodeData = [];
  nodesAfter.map((item) => {
    allNodeData.push(item.id);
  });
  // 如果allNodeDate包含code则删除
  if (allNodeData.includes(code)) {
    allNodeData.splice(allNodeData.indexOf(code), 1);
  }
  let allNodes = nodeLib.reduce((r, i) => r.concat(i.nodes), []);
  nodeList.forEach((item) => {
    if (allNodeData.includes(item.code)) {
      // 重置节点数据
      let node = allNodes.find((i) => i.name === item.name);

      item.configJson = JSON.parse(JSON.stringify(node.configJson));
      item.extJson.columns = [];
      item.extJson.status = 0;
      if (item.extJson?.fieldConfig) {
        item.extJson.fieldConfig = [];
      }
      // 如果extJson里有mapListData， fieldList则清空
      if (item.extJson?.mapListData) {
        item.extJson.mapListData = [];
      }
      if (item.extJson?.fieldList) {
        item.extJson.fieldList = [];
      }
    }
  });
};

// 设置下节点信息
export const setNextNode = (nodeList, edges, node, type = 'set') => {
  // 遍历 edges找到 node的下一个节点
  let fromCode = [];
  edges.forEach((item) => {
    if (item.source === node.code) {
      fromCode.push(item.target);
    }
  });
  // 如果调用该方法则会为下个节点添加需要更新的状态
  if (fromCode.length > 0) {
    nodeList.forEach((item) => {
      if (fromCode.includes(item.code)) {
        // 为下级节点传递新的 oldColumnsKeys 用于校验更新
        if (type === 'set') item.extJson.status = -2;
        // 上个节点的数据
        item.extJson.oldColumnsKeys = node.extJson.columns.map((item) => item.columnName);
      }
    });
  }
};
// 对比节点信息状态
export const checkNode = (nodeList, edges, node) => {
  // 获取当前columns name
  if (node.extJson.status !== -2) return;
  // 让下个节点有当前节点的数据
  setNextNode(nodeList, edges, node);
  let columnsNameList = node.extJson.columns.map((item) => item.columnName);
  if (columnsNameList.length === 0) {
    return;
  }
  // 从上个节点传来的数据
  // 获取上个节点的oldColumnsKeys
  // let nodeEdges = edges.find((item) => item.target === node.code);
  // if (!nodeEdges) return;
  // let nodeOld = nodeList.find((item) => item.code === nodeEdges.source);
  // if (!node.extJson?.oldColumnsKeys || node.extJson.oldColumnsKeys.length === 0) {
  //   node.extJson.oldColumnsKeys = nodeOld?.extJson.oldColumnsKeys || [];
  // }
  let oldColumnsKeys = node.extJson.oldColumnsKeys || [];
  //  对比oldColumnskeys是否包含columnsNameList
  let flag = columnsNameList.every((item) => oldColumnsKeys.includes(item));
  // 如果不包含则更新下个节点
  
  if (flag) return;
  ElMessage({
    message: "父节点数据存在变更，请重新获取数据",
    type: "warning",
    center: true,
    grouping: true,
    duration: 5000,
    showClose: true,
  });
  let fromCode = [];
  edges.forEach((item) => {
    if (item.source === node.code) {
      fromCode.push(item.target);
    }
  });
  if (fromCode.length > 0) {
    setNextNode(nodeList, edges, node);
  }
};
// 重置当前节点configJson
//  参数 node
export const resetNodeConfig = (node) => {
  let allNodes = nodeLib.reduce((r, i) => r.concat(i.nodes), []);
  let nodeConfig = allNodes.find((item) => item.name === node.name);
  let data = JSON.parse(JSON.stringify(nodeConfig.configJson));
  Object.keys(data).forEach((key) => {
    node.configJson[key] = data[key];
  });
  node.extJson.oldColumnsKeys = node.extJson.columns.map((item) => item.columnName);
};
// 如何确保下个节点有当前节点的数据