import axios from 'axios';
import {ElMessage} from 'element-plus';
import {getToken} from './auth'
import setting from './setting'
import store from '@/store';

axios.defaults.baseURL = setting.baseUrl
// 跨域cookie
axios.defaults.withCredentials = true
//默认超时时间
axios.defaults.timeout = 0;
//返回其他状态码
axios.defaults.validateStatus = function (status) {
  return status >= 200 && status <= 500;
};
//跨域请求，允许保存cookie
axios.defaults.withCredentials = true;

axios.interceptors.request.use(
  config => {
    config.headers['Blade-Auth'] = 'bearer ' + getToken()
    return config;
  },
  error => {
    return Promise.reject(error);
  }
);
let defaultMsg = "服务器错误，请稍后再试"
axios.interceptors.response.use(
  response => {
    const res = response.data;
    if (res.code === 401) {
      // 生产环境跳转主项目登录获取token
      setTimeout(() => {
        if (process.env.NODE_ENV !== "development") {
          location.href = location.origin + '/#/login'
        }
      }, 300)
      ElMessage({message: res.msg, type: 'error', duration: 5000});
      return Promise.reject(new Error(res.msg || 'Error'));
    }
    if (res.code !== 200) {
      let dataMsg = res.data != null ? res.data : "";
      if(res.code == 400 ) dataMsg = ''
      // dataMsg 长度超出40并且处于job/edit页面
      // if(res.msg.length > 20 && location.hash.indexOf('job/edit') > -1)  {
      //   console.log(res);
      //   store.commit('nodeError/setNodeError', res.msg || res.data)
      //   // 请前往错误日志查看
      //   ElMessage({
      //     message: `请前往错误日志查看`,
      //     type: 'error',
      //   })
      // }else {
        ElMessage({
            offset: window.innerHeight / 5 - 50,
            showClose: true,
            dangerouslyUseHTMLString: true,
            message: `<div style="font-size:14px;min-height:30px;max-height:500px;overflow-y:auto;line-height:30px" title="${dataMsg}">${res.msg + dataMsg}</div>`,
            duration: 10000,
            type: 'error'
        });
      // }
      // ElMessage({message: res.msg + res.data || defaultMsg, type: 'error', duration: 5000});
      return Promise.reject(new Error(res.msg || defaultMsg));
    }
    return res
  },
  res => {
    ElMessage({message: defaultMsg, type: 'error', duration: 2000});
    return Promise.reject(new Error(defaultMsg));
  }
);

export default axios
